@@ -84,9 +84,9 @@ int fd_pair(int fd[2])
 }
 
 
-void print_child_argv(char **cmd)
+static void print_child_argv(char **cmd)
 {
-	rprintf(FINFO, RSYNC_NAME ": open connection using ");
+	rprintf(FINFO, "opening connection using ");
 	for (; *cmd; cmd++) {
 		/* Look for characters that ought to be quoted.  This
 		* is not a great quoting algorithm, but it's
@@ -121,7 +121,7 @@ pid_t piped_child(char **command, int *f_in, int *f_out)
 	int from_child_pipe[2];
 	extern int blocking_io;
 	
-	if (verbose > 0) {
+	if (verbose >= 2) {
 		print_child_argv(command);
 	}
 
@@ -240,10 +240,18 @@ void overflow(char *str)
 
 
 
-int set_modtime(char *fname,time_t modtime)
+int set_modtime(char *fname, time_t modtime)
 {
 	extern int dry_run;
-	if (dry_run) return 0;
+	if (dry_run)
+		return 0;
+
+	if (verbose > 2) {
+		rprintf(FINFO, "set modtime of %s to (%ld) %s",
+			fname, (long) modtime,
+			asctime(localtime(&modtime)));
+	}
+	
 	{
 #ifdef HAVE_UTIMBUF
 		struct utimbuf tbuf;  
